package org.openxma.dsl.core.model;


/**
 * Represents a callback interface used to augment model elements before they
 * are getting processed by templates and after any execution of model
 * validators.
 * 
 * Implementations of this interface use this to implement any project specific
 * defaults like the addition of various standard attributes required for this
 * domain.
 */
public interface ModelModifier {
	/**
	 * Callback method to augment the given <code>Model</code> instance.
	 * 
	 * @param model
	 *            the Model to augment
	 */
	void modifyModel(Model model);
	/**
	 * Provider that associates a <code>ModelModifier</code> instance with the current thread
	 */
	class Holder {
		private static final ThreadLocal<ModelModifier> modelModifierHolder = new ThreadLocal<ModelModifier>();

		/**
		 * Reset the ModelModifier for the current thread.
		 */
		public static void resetModelModifier() {
			modelModifierHolder.set(null);
		}

		/**
		 * Associate the given ModelModifier with the current thread, <i>not</i>
		 * exposing it as inheritable for child threads.
		 * 
		 * @param modelModifier
		 *            the current ModelModifier, or <code>null</code> to reset
		 *            the thread-bound context
		 */
		public static void setModelModifier(ModelModifier modelModifier) {
			modelModifierHolder.set(modelModifier);
		}

		/**
		 * Return the ModelModifier associated with the current thread, if any.
		 * 
		 * @return the current ModelModifier
		 */
		public static ModelModifier getModelModifier() {
			return modelModifierHolder.get();
		}
	}
}
